#include "kbiff.h"
#include "version.h"
#include <KWindowSystem>
#include <KApplication>
#include <klocale.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <KLocalizedString>
#include "setupdlg.h"

static const char *description = 
    I18N_NOOP("Full featured mail notification utility.");

KCmdLineOptions get_options()
{
    KCmdLineOptions options;
    options.add("secure", ki18n("Run in secure mode"));
    options.add("profile <profile>", ki18n("Use 'profile'"));
    return options;
}

extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
{
    KAboutData aboutData( "kbiff", QByteArray(),
                          ki18n("KBiff"),
                          kbiff_version, 
                          ki18n(description), 
                          KAboutData::License_GPL,
                          ki18n("(c) 1998-2008, Kurt Granroth"));
    aboutData.addAuthor(ki18n("Kurt Granroth"), ki18n("Author"), "granroth@kde.org");
    KCmdLineArgs::init( argc, argv, &aboutData );
    KCmdLineArgs::addCmdLineOptions(get_options());

    KApplication app;
    KBiff kbiff;
    KBiffSetup* setup = 0;
    bool is_secure    = false;
    bool have_profile = false;
    QString profile;

    KIconLoader kiconLoader("kbiff");    
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
    is_secure = args->isSet("secure");

    if (args->isSet("profile"))
    {
        profile = args->getOption("profile");
        have_profile = true;
    }

    args->clear();

    // restore this app if it is
    if (kapp->isSessionRestored())
        kbiff.readSessionConfig();
    else
    {
        // do we have the profile option?
        if (have_profile)
        {
            setup = new KBiffSetup(profile, is_secure);
        }
        else
        {
            setup = new KBiffSetup();

            if (!setup->exec())
            {
                delete setup;
                return 0;
            }
        }
        kbiff.processSetup(setup, true);

    }

    // check if we are docked (only if restored)
    if (kbiff.isDocked())
    {
        kapp->setTopWidget(new QWidget);
    }
    else
        kapp->setTopWidget(&kbiff);

    app.setQuitOnLastWindowClosed(false);
    return app.exec();
}
